@@ -4,6 +4,7 @@
  * Copyright (C) 2003 Jacob Perkins
  * Copyright (C) 2004 Stefan Walter
  * Copyright (C) 2005 Adam Schreiber
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,15 +32,17 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 
+#include "seahorse-object-list.h"
+#include "seahorse-progress.h"
 #include "seahorse-util.h"
 
-#include "common/seahorse-object-list.h"
-
 #include "pgp/seahorse-gpgme.h"
 #include "pgp/seahorse-gpgme-data.h"
 #include "pgp/seahorse-gpg-op.h"
 #include "pgp/seahorse-gpgme-key-op.h"
-#include "pgp/seahorse-gpgme-operation.h"
+
+#define DEBUG_FLAG SEAHORSE_DEBUG_KEYS
+#include "seahorse-debug.h"
 
 #define PROMPT "keyedit.prompt"
 #define QUIT "quit"
@@ -50,27 +53,56 @@
 #define PRINT(args)  if(!seahorse_util_print_fd args) return GPG_E (GPG_ERR_GENERAL)
 #define PRINTF(args) if(!seahorse_util_printf_fd args) return GPG_E (GPG_ERR_GENERAL)
 
- 
-#ifndef DEBUG_OPERATION_ENABLE
-#if _DEBUG
-#define DEBUG_OPERATION_ENABLE 1
-#else
-#define DEBUG_OPERATION_ENABLE 0
-#endif
-#endif
-
-#if DEBUG_OPERATION_ENABLE
-#define DEBUG_OPERATION(x) g_printerr x
-#else
-#define DEBUG_OPERATION(x) 
-#endif
-
 #define GPG_UNKNOWN     1
 #define GPG_NEVER       2
 #define GPG_MARGINAL    3
 #define GPG_FULL        4
 #define GPG_ULTIMATE    5
 
+typedef struct {
+	GCancellable *cancellable;
+	gpgme_ctx_t gctx;
+} key_op_generate_closure;
+
+static void
+key_op_generate_free (gpointer data)
+{
+	key_op_generate_closure *closure = data;
+	g_clear_object (&closure->cancellable);
+	gpgme_release (closure->gctx);
+	g_free (closure);
+}
+
+static void
+on_key_op_generate_progress (void *opaque,
+                             const char *what,
+                             int type,
+                             int current,
+                             int total)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (opaque);
+	key_op_generate_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	seahorse_progress_update (closure->cancellable, res, "%s", what);
+}
+
+static gboolean
+on_key_op_generate_complete (gpgme_error_t gerr,
+                             gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	key_op_generate_closure *closure = g_simple_async_result_get_op_res_gpointer (res);
+	GError *error = NULL;
+
+	if (seahorse_gpgme_propagate_error (gerr, &error)) {
+		g_simple_async_result_take_error (res, error);
+	}
+
+	seahorse_progress_end (closure->cancellable, res);
+	g_simple_async_result_complete (res);
+	return FALSE; /* don't call again */
+}
+
+
 /**
  * seahorse_gpgme_key_op_generate:
  * @sksrc: #SeahorseSource
@@ -81,88 +113,126 @@
  * @type: Key type. Supported types are #DSA_ELGAMAL, #DSA, #RSA_SIGN, and #RSA_RSA
  * @length: Length of key, must be within the range of @type specified by #SeahorseKeyLength
  * @expires: Expiration date of key
- * @err: Catches errors in the params
  * 
  * Tries to generate a new key based on given parameters.
- *
- * Returns: SeahorseOperation*
  **/
-SeahorseOperation*
-seahorse_gpgme_key_op_generate (SeahorseGpgmeSource *psrc, const gchar *name,
-                                const gchar *email, const gchar *comment,
-                                const gchar *passphrase, const SeahorseKeyEncType type,
-                                const guint length, const time_t expires, gpgme_error_t *err)
+void
+seahorse_gpgme_key_op_generate_async (SeahorseGpgmeSource *source,
+                                      const gchar *name,
+                                      const gchar *email,
+                                      const gchar *comment,
+                                      const gchar *passphrase,
+                                      const SeahorseKeyEncType type,
+                                      const guint length,
+                                      const time_t expires,
+                                      GCancellable *cancellable,
+                                      GAsyncReadyCallback callback,
+                                      gpointer user_data)
 {
-    SeahorseGpgmeOperation *pop = NULL;
-    gchar *common, *key_type, *start, *expires_date;
-    const gchar *parms;
-    
-    *err = GPG_OK;
+	gchar *common, *key_type, *start, *expires_date;
+	key_op_generate_closure *closure;
+	GSimpleAsyncResult *res;
+	GError *error = NULL;
+	const gchar *parms;
+	gpgme_error_t gerr;
+	GSource *gsource;
 
-    if (strlen (name) < 5)
-        *err = GPG_E (GPG_ERR_INV_VALUE);
+	g_return_if_fail (SEAHORSE_IS_GPGME_SOURCE (source));
+	g_return_if_fail (name);
+	g_return_if_fail (strlen (name) > 4);
+	g_return_if_fail (passphrase);
 
-    /* Check lengths for each type */
-    switch (type) {
-    case DSA_ELGAMAL:
-        if (length < ELGAMAL_MIN || length > LENGTH_MAX)
-            *err = GPG_E (GPG_ERR_INV_VALUE);
-        break;
-    case DSA:
-        if (length < DSA_MIN || length > DSA_MAX)
-            *err = GPG_E (GPG_ERR_INV_VALUE);
-        break;
-    case RSA_RSA:
-    case RSA_SIGN:
-        if (length < RSA_MIN || length > LENGTH_MAX)
-            *err = GPG_E (GPG_ERR_INV_VALUE);
-        break;
-    default:
-        *err = GPG_E (GPG_ERR_INV_VALUE);
-        break;
-    }
+	/* Check lengths for each type */
+	switch (type) {
+	case DSA_ELGAMAL:
+		g_return_if_fail (length >= ELGAMAL_MIN || length <= LENGTH_MAX);
+		break;
+	case DSA:
+		g_return_if_fail (length >= DSA_MIN || length <= DSA_MAX);
+		break;
+	case RSA_RSA:
+	case RSA_SIGN:
+		g_return_if_fail (length >= RSA_MIN || length <= LENGTH_MAX);
+		break;
+	default:
+		g_return_if_reached ();
+		break;
+	}
 
-    if (0 != expires)
-        expires_date = seahorse_util_get_date_string (expires);
-    else
-        expires_date = g_strdup ("0");
+	if (expires != 0)
+		expires_date = seahorse_util_get_date_string (expires);
+	else
+		expires_date = g_strdup ("0");
 
-    /* Common xml */
-    common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
-                              "</GnupgKeyParms>", name, expires_date, passphrase);
-    if (email != NULL && strlen (email) > 0)
-        common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
-    if (comment != NULL && strlen (comment) > 0)
-        common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
+	/* Common xml */
+	common = g_strdup_printf ("Name-Real: %s\nExpire-Date: %s\nPassphrase: %s\n"
+			"</GnupgKeyParms>", name, expires_date, passphrase);
+	if (email != NULL && strlen (email) > 0)
+		common = g_strdup_printf ("Name-Email: %s\n%s", email, common);
+	if (comment != NULL && strlen (comment) > 0)
+		common = g_strdup_printf ("Name-Comment: %s\n%s", comment, common);
 
-    if (type == DSA || type == DSA_ELGAMAL)
-        key_type = "Key-Type: DSA\nKey-Usage: sign";
-    else
-        key_type = "Key-Type: RSA\nKey-Usage: sign";
+	if (type == DSA || type == DSA_ELGAMAL)
+		key_type = "Key-Type: DSA\nKey-Usage: sign";
+	else
+		key_type = "Key-Type: RSA\nKey-Usage: sign";
 
-    start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
+	start = g_strdup_printf ("<GnupgKeyParms format=\"internal\">\n%s\nKey-Length: ", key_type);
 
-    /* Subkey xml */
-    if (type == DSA_ELGAMAL)
-        parms = g_strdup_printf ("%s%d\nSubkey-Type: ELG-E\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
-                                 start, (length < DSA_MAX) ? length : DSA_MAX, length, common);
-    else if (type == RSA_RSA)
-        parms = g_strdup_printf ("%s%d\nSubkey-Type: RSA\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
-                                 start, length, length, common);
-    else
-        parms = g_strdup_printf ("%s%d\n%s", start, length, common);
+	/* Subkey xml */
+	if (type == DSA_ELGAMAL)
+		parms = g_strdup_printf ("%s%d\nSubkey-Type: ELG-E\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
+		                         start, (length < DSA_MAX) ? length : DSA_MAX, length, common);
+	else if (type == RSA_RSA)
+		parms = g_strdup_printf ("%s%d\nSubkey-Type: RSA\nSubkey-Length: %d\nSubkey-Usage: encrypt\n%s",
+		                         start, length, length, common);
+	else
+		parms = g_strdup_printf ("%s%d\n%s", start, length, common);
 
-    if (GPG_IS_OK (*err)) {
-        pop = seahorse_gpgme_operation_new (NULL);
-        *err = gpgme_op_genkey_start (pop->gctx, parms, NULL, NULL);
-     }
+	res = g_simple_async_result_new (G_OBJECT (source), callback, user_data,
+	                                 seahorse_gpgme_key_op_generate_async);
 
-    /* Free xmls */
-    g_free (start);
-    g_free (common);
-    g_free (expires_date);
+	closure = g_new0 (key_op_generate_closure, 1);
+	closure->gctx = seahorse_gpgme_source_new_context ();
+	closure->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+	gpgme_set_progress_cb (closure->gctx, on_key_op_generate_progress, res);
+	g_simple_async_result_set_op_res_gpointer (res, closure, key_op_generate_free);
 
-    return pop ? SEAHORSE_OPERATION (pop) : NULL;
+	seahorse_progress_prep_and_begin (cancellable, res, NULL);
+	gsource = seahorse_gpgme_gsource_new (closure->gctx, cancellable);
+	g_source_set_callback (gsource, (GSourceFunc)on_key_op_generate_complete,
+	                       g_object_ref (res), g_object_unref);
+
+	gerr = gpgme_op_genkey_start (closure->gctx, parms, NULL, NULL);
+	if (seahorse_gpgme_propagate_error (gerr, &error)) {
+		g_simple_async_result_take_error (res, error);
+		g_simple_async_result_complete_in_idle (res);
+	} else {
+		g_source_attach (gsource, g_main_context_default ());
+	}
+
+	g_source_unref (gsource);
+
+	/* Free xmls */
+	g_free (start);
+	g_free (common);
+	g_free (expires_date);
+
+	g_object_unref (res);
+}
+
+gboolean
+seahorse_gpgme_key_op_generate_finish (SeahorseGpgmeSource *source,
+                                       GAsyncResult *result,
+                                       GError **error)
+{
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (source),
+	                      seahorse_gpgme_key_op_generate_async), FALSE);
+
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result), error))
+		return FALSE;
+
+	return TRUE;
 }
 
 /* helper function for deleting @skey */
@@ -257,10 +327,10 @@ seahorse_gpgme_key_op_edit (gpointer data, gpgme_status_code_t status,
 	    status == GPGME_STATUS_SIGEXPIRED || status == GPGME_STATUS_KEYEXPIRED ||
 	    status == GPGME_STATUS_PROGRESS || status == GPGME_STATUS_KEY_CREATED ||
 	    status == GPGME_STATUS_ALREADY_SIGNED || status == GPGME_STATUS_MISSING_PASSPHRASE)		
-        return parms->err;
-    
-    DEBUG_OPERATION (("[edit key] state: %d / status: %d / args: %s\n", 
-                      parms->state, status, args));
+		return parms->err;
+
+	seahorse_debug ("[edit key] state: %d / status: %d / args: %s",
+	                parms->state, status, args);
 
 	/* Choose the next state based on the current one and the input */
 	parms->state = parms->transit (parms->state, status, args, parms->data, &parms->err);
@@ -834,9 +904,9 @@ seahorse_gpgme_key_op_set_trust (SeahorseGpgmeKey *pkey, SeahorseValidity trust)
 {
 	SeahorseEditParm *parms;
 	gint menu_choice;
-	
-	DEBUG_OPERATION(("[GPGME_KEY_OP] set_trust: trust = %i",trust));
-	
+
+	seahorse_debug ("[GPGME_KEY_OP] set_trust: trust = %i", trust);
+
 	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	g_return_val_if_fail (trust >= SEAHORSE_VALIDITY_NEVER, GPG_E (GPG_ERR_INV_VALUE));
 	g_return_val_if_fail (seahorse_gpgme_key_get_trust (pkey) != trust, GPG_E (GPG_ERR_INV_VALUE));
@@ -2394,7 +2464,7 @@ photoid_load_transit (guint current_state, gpgme_status_code_t status,
 	case PHOTO_ID_LOAD_DESELECT:
 		if (parm->uid < parm->num_uids) {
 			parm->uid = parm->uid + 1;
-			DEBUG_OPERATION (("PhotoIDLoad Next UID %i\n", parm->uid));
+			seahorse_debug ("PhotoIDLoad Next UID %i", parm->uid);
 			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT)) {
 				next_state = PHOTO_ID_LOAD_SELECT;
 			} else {
@@ -2404,7 +2474,7 @@ photoid_load_transit (guint current_state, gpgme_status_code_t status,
 		} else {
 			if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT)) {
 				next_state = PHOTO_ID_LOAD_QUIT;
-				DEBUG_OPERATION (("PhotoIDLoad Quiting Load\n"));
+				seahorse_debug ("PhotoIDLoad Quiting Load");
 			} else {
 				*err = GPG_E (GPG_ERR_GENERAL);
 				g_return_val_if_reached (PHOTO_ID_LOAD_ERROR);
@@ -2415,7 +2485,7 @@ photoid_load_transit (guint current_state, gpgme_status_code_t status,
 	case PHOTO_ID_LOAD_QUIT:
 		/* Shouldn't be reached */
 		*err = GPG_E (GPG_ERR_GENERAL);
-		DEBUG_OPERATION (("PhotoIDLoad Reached Quit\n"));
+		seahorse_debug ("PhotoIDLoad Reached Quit");
 		g_return_val_if_reached (PHOTO_ID_LOAD_ERROR);
 		break;
 		
@@ -2449,9 +2519,9 @@ seahorse_gpgme_key_op_photos_load (SeahorseGpgmeKey *pkey)
 	g_return_val_if_fail (key, GPG_E (GPG_ERR_INV_VALUE));
 	g_return_val_if_fail (key->subkeys && key->subkeys->keyid, GPG_E (GPG_ERR_INV_VALUE));
 	keyid = key->subkeys->keyid;
-	
-	DEBUG_OPERATION (("PhotoIDLoad Start\n"));
-    
+
+	seahorse_debug ("PhotoIDLoad Start");
+
 	fd = g_mkstemp (image_path);
 	if(fd == -1) { 
 		gerr = GPG_E(GPG_ERR_GENERAL);
@@ -2467,11 +2537,11 @@ seahorse_gpgme_key_op_photos_load (SeahorseGpgmeKey *pkey)
 		photoid_load_parm.photos = NULL;
 		photoid_load_parm.output_file = image_path;
 		photoid_load_parm.key = key;
-        
-		DEBUG_OPERATION (("PhotoIdLoad KeyID %s\n", keyid));
+
+		seahorse_debug ("PhotoIdLoad KeyID %s", keyid);
 		gerr = seahorse_gpg_op_num_uids (NULL, keyid, &(photoid_load_parm.num_uids));
-		DEBUG_OPERATION (("PhotoIDLoad Number of UIDs %i\n", photoid_load_parm.num_uids));
-        
+		seahorse_debug ("PhotoIDLoad Number of UIDs %i", photoid_load_parm.num_uids);
+
 		if (GPG_IS_OK (gerr)) {
             
 			setenv ("SEAHORSE_IMAGE_FILE", image_path, 1);
@@ -2494,9 +2564,9 @@ seahorse_gpgme_key_op_photos_load (SeahorseGpgmeKey *pkey)
 		
 		seahorse_object_list_free (photoid_load_parm.photos);
 	}
-    
-	DEBUG_OPERATION (("PhotoIDLoad Done\n"));
-    
+
+	seahorse_debug ("PhotoIDLoad Done");
+
 	return gerr;
 }
 
