@@ -2,7 +2,8 @@
  * Seahorse
  * 
  * Copyright (C) 2008 Stefan Walter
- * 
+ * Copyright (C) 2011 Collabora Ltd.
+ *
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
@@ -26,9 +27,8 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
-#include <seahorse-viewer.h>
-#include <seahorse-operation.h>
-#include <seahorse-object.h>
+#include "seahorse-viewer.h"
+#include "seahorse-object.h"
 
 G_BEGIN_DECLS
 
@@ -55,8 +55,7 @@ struct _SeahorseKeyserverResultsClass {
 
 GType            seahorse_keyserver_results_get_type         (void);
 
-void             seahorse_keyserver_results_show             (SeahorseOperation* op, GtkWindow* parent, 
-                                                              const char* search_text);
+void             seahorse_keyserver_results_show             (const gchar *search_text);
 
 const gchar*     seahorse_keyserver_results_get_search       (SeahorseKeyserverResults* self);
 
